// Variable to store live wallpaper parameters object
lw_params_config = 0;

enum SCALE_MODE
{
	// Keep the wallpaper the same size as long as it covers the entire
	// window. If the window is larger than the wallpaper, then the
	// wallpaper is scaled up while keeping the aspect ratio.
	NO_SCALE,
	// Scale the wallpaper to cover the entire window, while keeping the
	// aspect ratio.
	SCALE,
	// Stretch the wallpaper over the entire window, ignoring the aspect
	// ratio.
	STRETCH,
	// (total number of options in this enum)
	SIZE
};

// Scaling factor for the application surface. Lower values increase rendering
// performance but decrease quality.
render_scale = 1.0;

// The scaling mode to use to make the wallpaper cover the entire window.
// Use values from the SCALE_MODE enum. Default value is SCALE_MODE.NO_SCALE.
scale_mode = SCALE_MODE.SCALE;

// The width of the wallpaper. Default value is the width of the room.
wallpaper_width = room_width;

// The height of the wallpaper. Default value is the height of the room.
wallpaper_height = room_height;

// The focus point on the X axis. Use values in range 0..1, where 0 is the left
// side of the wallpaper and 1 is the right side. Default value is 0.5 (center).
focus_x = 0.5;

// The focus point on the Y axis. Use values in range 0..1, where 0 is the top
// of the wallpaper and 1 is the bottom. Default value is 0.5 (center).
focus_y = 0.5;

// Variable used for audio segments
segment_count = 4;

// Variable used to identify if debug info should be shown
show_debug = false;

// Variable used for maximum ducks allowed
duck_count = 20;

// Add configuration options for game speed and render resolution and subscription metrics
var _config =
[
	{
		type: "section",
		name: "perf",
		label: "Performance",
		children:
		[
			{
				type: "select",
				name: "lw_speed",
				label: "Speed",
				value: "Blazing",
				options: ["Low", "Normal", "Blazing"],
				description:"Performance option",
			},
			{
				type: "select",
				name: "lw_resolution",
				label: "Resolution",
				value: "Crisp",
				options: ["Blurry", "Normal", "Crisp"],
				description: "Performance option",
			}
		]
	},
	{
		type: "section",
		name: "subs",
		label: "Metrics",
		children:
		[
			{
				type: "boolean",
				name: "cpu",
				label: "Allow CPU Data",
				value: true
			},
			{
				type: "boolean",
				name: "gpu",
				label: "Allow GPU Data",
				value: true
			},
			{
				type: "boolean",
				name: "battery",
				label: "Allow Battery Data",
				value: true
			},
			{
				type: "boolean",
				name: "ram",
				label: "Allow RAM Data",
				value: true
			},
			{
				type: "boolean",
				name: "disk",
				label: "Allow Disk Data",
				value: true
			},
			{
				type: "boolean",
				name: "network",
				label: "Allow Network Data",
				value: true
			},
			{
				type: "boolean",
				name: "audio",
				label: "Allow Audio Data",
				value: true
			},
			{
				type: "boolean",
				name: "mouse",
				label: "Allow Mouse Data",
				value: true
			}
		]
	},
	{
		type: "section",
		name: "opti",
		label: "Scene Options",
		children:
		[
			{
				type: "boolean",
				name: "debug_info",
				label: "Show Debug Info",
				value: false
			},
			{
				type: "boolean",
				name: "is_water",
				label: "Show Water",
				value: true
			},
			{
				type: "range",
				name: "audio_segments",
				label: "Water Segments",
				value: 4,
				min: 4,
                max: 128
			},
			{
				type: "range",
				name: "max_ducks",
				label: "Maximum Ducks",
				value: 12,
				min: 0,
                max: 20
			}
		]
	}
];

// Function for updating config settings
update_config = function()
{
	switch (lw_params_config.perf.lw_speed)
	{
		case "Blazing":
			game_set_speed(60, gamespeed_fps);
			break;

		case "Normal":
			game_set_speed(30, gamespeed_fps);
			break;

		case "Low":
			game_set_speed(20, gamespeed_fps);
			break;

		default:
			break;
	}

	switch (lw_params_config.perf.lw_resolution)
	{
		case "Crisp":
			render_scale = 1.0;
			break;

		case "Normal":
			render_scale = 0.5
			break;

		case "Blurry":
			render_scale = 0.25;
			break;

		default:
			break;
	}
	
	// Creates new emepty subs config array
	var _new_subs_config = [];
	
	if (lw_params_config.subs.cpu)
	{
		array_push(_new_subs_config, "cpu");
	}

	if (lw_params_config.subs.gpu)
	{
		array_push(_new_subs_config, "gpu");
	}

	if (lw_params_config.subs.battery)
	{
		array_push(_new_subs_config, "battery");
	}

	if (lw_params_config.subs.ram)
	{
		array_push(_new_subs_config, "ram");
	}

	if (lw_params_config.subs.disk)
	{
		array_push(_new_subs_config, "disk");
	}

	if (lw_params_config.subs.network)
	{
		array_push(_new_subs_config, "network");
	}

	if (lw_params_config.subs.audio)
	{
		array_push(_new_subs_config, "audio");
	}

	if (lw_params_config.subs.mouse)
	{
		array_push(_new_subs_config, "desktop_mouse");
	}

	// Sets wallpaper subscriptions to new config
	wallpaper_set_subscriptions(_new_subs_config);
	
	// Checks live wallpaper handler exists
	if (!instance_exists(obj_lw_handler))
	{
		// Creates the LW handler
		var _lw_handler = instance_create_depth(0, 0, 0, obj_lw_handler);
	}
	
	// Resets the handler object
	obj_lw_handler.lw_subs_data = -1;
	
	// Sets variables from wallpaper config
	show_debug = lw_params_config.opti.debug_info;
	duck_count = lw_params_config.opti.max_ducks;
	segment_count = lw_params_config.opti.audio_segments;
	
	// Destroys all game objects (done to reset scene)
	
	with(obj_duck)
	{
		instance_destroy();
	}
	
	with(obj_water_segment)
	{
		instance_destroy();
	}
	
	with(obj_audio_bar)
	{
		instance_destroy();
	}
	
	with(obj_audio_visualiser)
	{
		instance_destroy();
	}
	
	with(obj_water_line)
	{
		instance_destroy();
	}
	
	// Check what kind of visualiser to create
	if (lw_params_config.opti.is_water)
	{	
		instance_create_layer(0, 0, "Water", obj_water_line);
	}
	else
	{
		instance_create_layer(0, 0, "Instances", obj_audio_visualiser);	
	}
}

wallpaper_set_config(_config);